<?php
/**
 * The dashboard-specific functionality of the plugin.
 *
 * @link       http://linkpizza.com
 * @since      1.0.0
 *
 * @package    linkpizza-manager
 * @subpackage linkpizza-manager/admin
 */

define('PZZ_OIDC_CLIENT_ID', 'linkpizza-manager-wordpress');
define('PZZ_OIDC_AUTH_URI', 'https://sso.linkpizza.com/auth/realms/linkpizza/protocol/openid-connect/auth');
define('PZZ_OIDC_TOKEN_ENDPOINT', 'https://sso.linkpizza.com/auth/realms/linkpizza/protocol/openid-connect/token');
define('PZZ_OIDC_HOST_HEADER', 'sso.linkpizza.com');
define('PZZ_OIDC_API_HOST_HEADER', 'api.pzz.io');
define('PZZ_OIDC_API_BASE_PATH', 'https://api.pzz.io/affiliate');
define('PZZ_OIDC_ACCESS_TOKEN_OPTION_NAME', 'pzz_oidc_access_token');
define('PZZ_OIDC_REFRESH_TOKEN_OPTION_NAME', 'pzz_oidc_refresh_token');


class linkPizza_Manager_Admin {


    /**
     *
     * Retrieve linkPizza information, now through openID
     *
     * Since 5.3
     *
     */
    function startOpenIDAuthentication (){
        $callbackUrl = admin_url( 'admin.php?page=linkpizza-manager&pzz_callback=oauth');
        $client = new OAuth2\Client(PZZ_OIDC_CLIENT_ID, '', OAuth2\Client::AUTH_TYPE_URI, ABSPATH . WPINC . '/certificates/ca-bundle.crt');
        if (!isset($_GET['code'])) {
            $auth_url = $client->getAuthenticationUrl(PZZ_OIDC_AUTH_URI, $callbackUrl);
            header('Location: ' . $auth_url);
            die('Redirect');
        }
    }

    function handleOpenIDCallback () {
        if(isset($_GET['code']) && isset($_GET['pzz_callback']) ) {
            $callbackUrl = admin_url( 'admin.php?page=linkpizza-manager&pzz_callback=oauth');
            $client = new OAuth2\Client(PZZ_OIDC_CLIENT_ID, '', OAuth2\Client::AUTH_TYPE_URI, ABSPATH . WPINC . '/certificates/ca-bundle.crt');
            $params = array('code' => $_GET['code'], 'redirect_uri' => $callbackUrl);
            $extraHeaders = array('Host' => PZZ_OIDC_HOST_HEADER);
            $response = $client->getAccessToken(PZZ_OIDC_TOKEN_ENDPOINT, 'authorization_code', $params, $extraHeaders);
            if ($response['code'] == 200 && isset($response["result"]) && isset($response["result"]["access_token"])) {
                $accessOidcToken = $response["result"]["access_token"];
                $refreshOidcToken = $response["result"]["refresh_token"];

                if (get_option(PZZ_OIDC_ACCESS_TOKEN_OPTION_NAME) !== false) {
                    update_option(PZZ_OIDC_ACCESS_TOKEN_OPTION_NAME, $accessOidcToken, 'yes');
                } else {
                    add_option(PZZ_OIDC_ACCESS_TOKEN_OPTION_NAME, $accessOidcToken, '', 'yes');
                }

                if (get_option(PZZ_OIDC_REFRESH_TOKEN_OPTION_NAME) !== false) {
                    update_option(PZZ_OIDC_REFRESH_TOKEN_OPTION_NAME, $refreshOidcToken, 'yes');
                } else {
                    add_option(PZZ_OIDC_REFRESH_TOKEN_OPTION_NAME, $refreshOidcToken, '', 'yes');
                }

                try {
                    $client->setAccessToken($accessOidcToken);
                    $client->setAccessTokenType(1);
                    $response = $client->fetch(PZZ_OIDC_API_BASE_PATH.'/user/me');
                    if($response['code'] == 200) {
                        $this->update_user_information( $response['result'] );
                        header('Location: ' . admin_url( 'admin.php?page=linkpizza-manager&pzz_message=login_success'));
                    }else{
                        header('Location: ' . admin_url( 'admin.php?page=linkpizza-manager&pzz_message=login_failure'));
                        die('Redirect');
                    }
                }catch (\OAuth2\Exception $e){
                    echo 'Caught exception: ',  $e->getMessage(), "\n";
                }
            } else{
                header('Location: ' . admin_url( 'admin.php?page=linkpizza-manager&pzz_message=login_failure'));
                die('Redirect');
            }
        }
    }

    function update_information($response){
	    $this->update_user_information( $response );
    }


	/**
	 * @param $response
	 */
	public function update_user_information( $response ) {
		if ( $response != null ) {
			if ( get_option( 'pzz_fullname' ) !== false ) {
				update_option( 'pzz_fullname', $response['fullName'], 'yes' );
			} else {
				add_option( 'pzz_fullname', $response['fullName'], '', 'yes' );
			}
			if ( get_option( 'pzz_username' ) !== false ) {
				update_option( 'pzz_username', $response['userName'], 'yes' );
			} else {
				add_option( 'pzz_username', $response['userName'], '', 'yes' );
			}
			if ( get_option( 'pzz_id' ) !== false ) {
				update_option( 'pzz_id', $response['id'], 'yes' );
			} else {
				add_option( 'pzz_id', $response['id'], '', 'yes' );
			}
		}
	}

	/**
	 *
	 * Display error message when LinkPizza can't find a valid API token.
	 *
	 * Since 4.5
	 *
	 */

	function pzz_admin_notice(){
        $accessOidcToken = get_option(PZZ_OIDC_ACCESS_TOKEN_OPTION_NAME);
        $refreshOidcToken = get_option(PZZ_OIDC_REFRESH_TOKEN_OPTION_NAME);
		if ($accessOidcToken == null || $refreshOidcToken == null || $accessOidcToken == "" || $refreshOidcToken == "") {
			?>
				<div class="error">
				    <p><?php _e( 'LinkPizza: You need to log in to activate the plugin: ', 'linkpizza-manager' ); ?>, <?php _e( 'please', 'linkpizza-manager' ); ?> <a href="../wp-admin/admin.php?page=linkpizza-manager"><?php _e( 'login here', 'linkpizza-manager' ); ?></a></p>
				</div>
			<?php
		}
        $this->showPzzMessage();
		$this->pzz_bulk_action_admin_notice();
	}

	function showPzzMessage()
    {
        if(isset($_GET['pzz_message'])) {
            if($_GET['pzz_message'] == 'login_success') {
            ?>
            <div class="update notice">
                <p><?php _e( 'You have succesfully logged in.', 'linkpizza-manager' ); ?></p>
            </div>
            <?php
             } else if($_GET['pzz_message'] == 'login_failure') {
            ?>
            <div class="error notice">
                <p><?php _e('Something went wrong while connecting to the LinkPizza API, the server returned an error. Please check with your hosting provider if connections to pzz.io are not blocked.', 'linkpizza-manager'); ?></p>
            </div>
            <?php
            }
        }
    }

	/**
	 *
	 * Adds settings field and sections for LinkPizza
	 *
	 * Since 4.5
	 *
	 */

	function pzz_settings_init(  ) {

		register_setting( 'linkPizza_Page', 'pzz_settings' );
		register_setting( 'linkPizza_Page', 'pzz_logged_in_user' );
		register_setting( 'linkPizza_Page', PZZ_OIDC_ACCESS_TOKEN_OPTION_NAME );
		register_setting( 'linkPizza_Page_advanced', 'pzz_tracking_only_categories' );
		register_setting( 'linkPizza_Page_advanced', 'pzz_tracking_only_on_posts_before' );
		register_setting( 'linkPizza_Page_advanced', 'pzz_indexable_javascript' );
		register_setting( 'linkPizza_Page_advanced', 'pzz_extend_preview_time' );
		register_setting( 'linkPizza_Page_link_summary', 'pzz_link_summary_border_color' );
		register_setting( 'linkPizza_Page_link_summary', 'pzz_link_summary_border_width' );
		register_setting( 'linkPizza_Page_link_summary', 'pzz_link_summary_border_padding' );
		register_setting( 'linkPizza_Page_link_summary', 'pzz_link_summary_width');
		register_setting( 'linkPizza_Page_link_summary', 'pzz_link_summary_link_color' );
		register_setting( 'linkPizza_Page_link_summary', 'pzz_link_summary_enabled' );
		register_setting( 'linkPizza_Page_link_summary', 'pzz_link_summary_position' );
		register_setting( 'linkPizza_Page_link_summary', 'pzz_link_summary_layout_type' );
		register_setting( 'linkPizza_Page_link_summary', 'pzz_link_summary_tag_background_color' );
		register_setting( 'linkPizza_Page_link_summary', 'pzz_link_summary_use_title_text' );


		add_settings_section(
			'pzz_linkPizza_Page_section',
			__( 'Login Credentials', 'linkpizza-manager' ),
			array( &$this, 'pzz_settings_section_callback'),
			'linkPizza_Page'
		);

		add_settings_section(
			'pzz_linkPizza_Page_section_link_summary',
			__( 'Link summary', 'linkpizza-manager' ),
			array( &$this, 'pzz_link_summary_settings_section_callback'),
			'linkPizza_Page_link_summary'
		);

		add_settings_section(
			'pzz_linkPizza_Page_section_advanced',
			__( 'Advanced', 'linkpizza-manager' ),
			array( &$this, 'pzz_advanced_settings_section_callback'),
			'linkPizza_Page_advanced'
		);

		add_settings_section(
			'pzz_linkPizza_Page_section_help',
			__( 'Help', 'linkpizza-manager' ),
			array( &$this, 'pzz_linkPizza_Page_section_help_callback'),
			'linkPizza_Page_help'
		);

		add_settings_field(
			'pzz_logged_in_user',
			__( 'Logged in user', 'linkpizza-manager' ),
			array( &$this, 'pzz_logged_in_user_render'),
			'linkPizza_Page',
			'pzz_linkPizza_Page_section'
		);

        add_settings_field(
            PZZ_OIDC_ACCESS_TOKEN_OPTION_NAME,
            __( 'Login', 'linkpizza-manager' ),
            array( &$this, 'pzz_oidc_token_render'),
            'linkPizza_Page',
            'pzz_linkPizza_Page_section'
        );


        add_settings_field(
			'pzz_tracking_only_categories',
			__( 'Disabled Categories', 'linkpizza-manager' ),
			array( &$this, 'pzz_tracking_only_categories_render'),
			'linkPizza_Page_advanced',
			'pzz_linkPizza_Page_section_advanced'
		);

		add_settings_field(
			'pzz_tracking_only_on_posts_before',
			__( 'Only convert links on posts published after (dd-mm-yyyy)', 'linkpizza-manager' ),
			array( &$this, 'pzz_tracking_only_on_posts_before_render'),
			'linkPizza_Page_advanced',
			'pzz_linkPizza_Page_section_advanced'
		);

		add_settings_field(
			'pzz_indexable_javascript',
			__( 'Indexable version of the script (Not recommended)', 'linkpizza-manager' ),
			array( &$this, 'pzz_indexable_javascript_render'),
			'linkPizza_Page_advanced',
			'pzz_linkPizza_Page_section_advanced'
		);

		add_settings_field(
			'pzz_extend_preview_time',
			__('extended preview time when using the <a href="https://wordpress.org/plugins/public-post-preview/">Public Post Preview</a> plugin', 'linkpizza-manager' ),
			array( &$this, 'pzz_extend_preview_time_render'),
			'linkPizza_Page_advanced',
			'pzz_linkPizza_Page_section_advanced'
		);

		//link summary

		add_settings_field(
			'pzz_link_summary_enabled',
			__( 'Enabled', 'linkpizza-manager' ),
			array( &$this, 'pzz_link_summary_enabled_render'),
			'linkPizza_Page_link_summary',
			'pzz_linkPizza_Page_section_link_summary'
		);

		add_settings_field(
			'pzz_link_summary_width',
			__( 'Width', 'linkpizza-manager' ),
			array( &$this, 'pzz_link_summary_width_render'),
			'linkPizza_Page_link_summary',
			'pzz_linkPizza_Page_section_link_summary'
		);

		add_settings_field(
			'pzz_link_summary_link_color',
			__( 'Link Color', 'linkpizza-manager' ),
			array( &$this, 'pzz_link_summary_link_color_render'),
			'linkPizza_Page_link_summary',
			'pzz_linkPizza_Page_section_link_summary'
		);

		add_settings_field(
			'pzz_link_summary_border_color',
			__( 'Border Color', 'linkpizza-manager' ),
			array( &$this, 'pzz_link_summary_border_color_render'),
			'linkPizza_Page_link_summary',
			'pzz_linkPizza_Page_section_link_summary'
		);
		add_settings_field(
			'pzz_link_summary_border_width',
			__( 'Border Width (put to 0 to hide border)', 'linkpizza-manager' ),
			array( &$this, 'pzz_link_summary_border_width_render'),
			'linkPizza_Page_link_summary',
			'pzz_linkPizza_Page_section_link_summary'
		);
		add_settings_field(
			'pzz_link_summary_border_padding',
			__( 'Border Padding', 'linkpizza-manager' ),
			array( &$this, 'pzz_link_summary_border_padding_render'),
			'linkPizza_Page_link_summary',
			'pzz_linkPizza_Page_section_link_summary'
		);
		add_settings_field(
			'pzz_link_summary_position',
			__( 'Position', 'linkpizza-manager' ),
			array( &$this, 'pzz_link_summary_position_render'),
			'linkPizza_Page_link_summary',
			'pzz_linkPizza_Page_section_link_summary'
		);
		add_settings_field(
			'pzz_link_summary_layout_type',
			__( 'Layout Type', 'linkpizza-manager' ),
			array( &$this, 'pzz_link_summary_layout_type_render'),
			'linkPizza_Page_link_summary',
			'pzz_linkPizza_Page_section_link_summary'
		);

		add_settings_field(
			'pzz_link_summary_tag_background_color',
			__( 'Tag Background color', 'linkpizza-manager' ),
			array( &$this, 'pzz_link_summary_tag_background_color_render'),
			'linkPizza_Page_link_summary',
			'pzz_linkPizza_Page_section_link_summary'
		);

		add_settings_field(
			'pzz_link_summary_tag_background_color',
			__( 'Tag Background color', 'linkpizza-manager' ),
			array( &$this, 'pzz_link_summary_tag_background_color_render'),
			'linkPizza_Page_link_summary',
			'pzz_linkPizza_Page_section_link_summary'
		);

		add_settings_field(
			'pzz_link_summary_use_title_text',
			__( 'Use title attribute as label, falls back to text on the link. If both are not there it uses the domain', 'linkpizza-manager' ),
			array( &$this, 'pzz_link_summary_use_title_text_render'),
			'linkPizza_Page_link_summary',
			'pzz_linkPizza_Page_section_link_summary'
		);

	}

    /**
     *  Listen for any button clicks
     *
     * @Since 5.3
     */
    function pzz_check_buttons_pressed () {
        $this->authenticate_button_pressed();
    }

    function authenticate_button_pressed (){
        if (isset($_POST['authenticate-linkpizza-shortcut']))
            if ($_POST['authenticate-linkpizza-shortcut'] == "submitted") {
                $this->startOpenIDAuthentication();
            }
    }


    /**
	 *
	 * Renders the input field for the API token and description below
	 *
	 * Since 4.5
	 *
	 */

	function pzz_logged_in_user_render()
	{

		?>
        <div id="pzz-preloaded-error-message">
            <p><?php _e('Something went wrong while the plugin tried to reach LinkPizza, please follow this <a href="https://www.elegantthemes.com/blog/tips-tricks/using-the-wordpress-debug-log">guide</a> to activate logging and send the log to us.', 'linkpizza-manager'); ?></p>
        </div>
		<?php
       $response = pzz_do_oauth_call_with_refresh_check(PZZ_OIDC_API_BASE_PATH.'/user/me', array(), 0);
		if ($response != null) {
		    $this->update_information($response);
			if (constant('OPENSSL_TLSEXT_SERVER_NAME') != true) {
				?>
				<div class="error">
					<p><?php _e('Your version of OpenSSL is outdated, please use at least version 0.9.8j to be able to use LinkPizza', 'linkpizza-manager'); ?></p>
				</div>
				<?php
			}
			?>
			<div>
				<p>
					<?php _e("Hi", 'linkpizza-manager'); ?> <b><?php echo $response['fullName']; ?></b>,

					<?php _e("Your LinkPizza installation is currently active under account number:", 'linkpizza-manager'); ?>
					<b><?php echo $response['id']; ?></b>
				</p>

			</div>
		<?php } else { ?>
			<div>
				<p>
					<?php _e("Your LinkPizza installation is currently inactive, login with the button below", 'linkpizza-manager'); ?>
				</p>
			</div>
			<?php
		}

		?>
        <script>
            document.getElementById("pzz-preloaded-error-message").style = "display:none;";
        </script>
		<?php
	}

    function pzz_oidc_token_render(  ) {
        //hack to avoid redirect
        ?>
        </form>
        <div>
            <form id="pzz_refresh_statistics" method='post'>
                <input type="submit" value="<?php _e('Authenticate with LinkPizza', 'linkpizza-manager' );?>" class="button">
                <input type="hidden" name="authenticate-linkpizza-shortcut" value="submitted">
            </form>
        </div>
        <?php
    }



    /**
	 *
	 * Renders the input field for the API token and description below
	 *
	 * Since 4.6
	 *
	 */


	function pzz_tracking_only_categories_render(  )
	{
		$trackingOnlyCategories = get_option('pzz_tracking_only_categories');
		$categories = get_categories();
		foreach ($categories as $category) { ?>
			<input class="widefat" type="checkbox" name="pzz_tracking_only_categories[<?php echo $category->cat_ID; ?>]"
				   value="<?php echo $category->cat_ID; ?>"
			<?php
			if (is_array($trackingOnlyCategories)) {
				echo (in_array($category->cat_ID, $trackingOnlyCategories)) ? 'checked="checked"' : ''; ?>  /> <?php
			} else {
				if ($category->cat_ID == $trackingOnlyCategories) {
					echo 'checked="checked" />';
				} else {
					echo '/>';
				}
			}
			?><span><?php echo $category->cat_name ?></span><br/><?php
		}
	}


	/**
	 *
	 * Renders the input field for the date after which posts should be enabled
	 *
	 * Since 5.2.3
	 *
	 */


    function pzz_tracking_only_on_posts_before_render(  )
    {
	    $trackingOnlyBeforeDate = get_option('pzz_tracking_only_on_posts_before');
	    ?>
            <script type="text/javascript">
                jQuery(document).ready(function($) {
                    $('.custom_date').datepicker({
                        changeMonth: true,
                        changeYear: true,
                        changeDay: true,
                        dateFormat : 'dd-mm-yy'
                    });
                });
            </script>
            <input type="text" class="custom_date" name="pzz_tracking_only_on_posts_before" value="<?php echo $trackingOnlyBeforeDate ?>"/>
	    <?php

    }

	/**
	 *
	 * Renders the checkbox for using the indexable version of the linkPizza script
	 *
	 * Since 5.0.6
	 *
	 */

	function pzz_indexable_javascript_render(  )
	{
		$indexableJavascript = get_option('pzz_indexable_javascript');
		?>
        <input class="widefat" type="checkbox" name="pzz_indexable_javascript" value="1" <?php echo checked($indexableJavascript, 1, true ) ?>  />
		<?php
	}

	/**
	 *
	 * Renders the checkbox for extending preview time when using the public-post-preview plugin
	 *
	 * Since 5.0.6
	 *
	 */

	function pzz_extend_preview_time_render(  )
	{
		$extendedPreviewTime = get_option('pzz_extend_preview_time');
		?>
        <input class="widefat" type="checkbox" name="pzz_extend_preview_time" value="1" <?php echo checked($extendedPreviewTime, 1, true ) ?>  />
		<?php
	}

	/**
	 * renders the Link Summary options
	 */

	function pzz_link_summary_width_render (){
		$pzz_link_summary_width = get_option('pzz_link_summary_width');
		?>
		<input type="number" name="pzz_link_summary_width"  min="0" max="100" value="<?php echo empty($pzz_link_summary_width) ? '100' : $pzz_link_summary_width ?>">%
		<?php
	}

	function pzz_link_summary_link_color_render (){
		$pzz_link_summary_link_color = get_option('pzz_link_summary_link_color');
		?>
		<input type="text" name="pzz_link_summary_link_color" class="pzz-colorpicker" value="<?php echo empty($pzz_link_summary_link_color) ? '#3699DC' : $pzz_link_summary_link_color?>">

		<?php
	}

	function pzz_link_summary_border_color_render (){
		$pzz_link_summary_border_color = get_option('pzz_link_summary_border_color');
		?>
		<input type="text" name="pzz_link_summary_border_color" class="pzz-colorpicker" value="<?php echo empty($pzz_link_summary_border_color) ? '#D3D3D3' : $pzz_link_summary_border_color?>">

		<?php
	}

	function pzz_link_summary_border_width_render (){
		$pzz_link_summary_border_width = get_option('pzz_link_summary_border_width');
		?>
		<input type="number" name="pzz_link_summary_border_width"  min="0" max="5" value="<?php echo empty($pzz_link_summary_border_width) ? '1' : $pzz_link_summary_border_width ?>">
		<?php
	}

	function pzz_link_summary_border_padding_render (){
		$pzz_link_summary_border_padding = get_option('pzz_link_summary_border_padding');
		?>
		<input type="number" name="pzz_link_summary_border_padding"  min="0" max="10" value="<?php echo empty($pzz_link_summary_border_padding) ? '6' : $pzz_link_summary_border_padding ?>">
		<?php
	}

	function pzz_link_summary_enabled_render(  )
	{
		$pzz_link_summary_enabled = get_option('pzz_link_summary_enabled');
		?>
		<input class="widefat" type="checkbox" name="pzz_link_summary_enabled" value="1" <?php echo checked($pzz_link_summary_enabled, 1, true ) ?>  />
		<?php
	}

	function pzz_link_summary_position_render(  )
	{
		$pzz_link_summary_position = get_option('pzz_link_summary_position');
		?>
		<input class="widefat" type="radio" name="pzz_link_summary_position" <?php if($pzz_link_summary_position == '1') echo 'checked="checked"'; ?> value="1" />Left
		<input class="widefat" type="radio" name="pzz_link_summary_position" <?php if($pzz_link_summary_position == '2') echo 'checked="checked"'; ?> value="2" />Center
		<input class="widefat" type="radio" name="pzz_link_summary_position" <?php if($pzz_link_summary_position == '3') echo 'checked="checked"'; ?> value="3" />Right
		<?php
	}

	function pzz_link_summary_layout_type_render(  )
	{
		$pzz_link_summary_layout_type = get_option('pzz_link_summary_layout_type');
		?>
		<input class="widefat" type="radio" name="pzz_link_summary_layout_type" <?php if($pzz_link_summary_layout_type == '1') echo 'checked="checked"'; ?> value="1" />List
		<input class="widefat" type="radio" name="pzz_link_summary_layout_type" <?php if($pzz_link_summary_layout_type == '2') echo 'checked="checked"'; ?> value="2" />Tags
		<?php
	}

	function pzz_link_summary_tag_background_color_render(  )
	{
		$pzz_link_summary_tag_background_color = get_option('pzz_link_summary_tag_background_color');
		?>
		<input type="text" name="pzz_link_summary_tag_background_color" class="pzz-colorpicker" value="<?php echo empty($pzz_link_summary_tag_background_color) ? '' : $pzz_link_summary_tag_background_color?>">
		<?php
	}

	function pzz_link_summary_use_title_text_render(  )
	{
		$pzz_link_summary_tag_background_color = get_option('pzz_link_summary_use_title_text');
		?>
		<input class="widefat" type="checkbox" name="pzz_link_summary_use_title_text" value="1" <?php echo checked($pzz_link_summary_tag_background_color, 1, true ) ?>  />
		<?php
	}

	/**
	 *
	 * Adds linkPizza option section description
	 *
	 * Since 4.5
	 *
	 */

	function pzz_settings_section_callback(  ) {
        _e( "Log in to your LinkPizza account to get started with this plugin, press the button below to continue  ");
	}

    /**
	 *
	 * Adds linkPizza disabled categories section description
	 *
	 * Since 4.6
	 *
	 */

	function pzz_advanced_settings_section_callback(  ) {
		_e( "LinkPizza will not change your links in these categories, this also disables monetization for those links", 'linkpizza-manager' );
	}

	/**
	 *
	 * Adds linkPizza disabled categories section description
	 *
	 * Since 4.6
	 *
	 */

	function pzz_top_link_ads_settings_section_callback(  ) {
		_e( "Configuration for the top links", 'linkpizza-manager' );
	}


	/**
	 *
	 * Adds linkPizza link summary description
	 *
	 * Since 5.0.0
	 *
	 */

	function pzz_link_summary_settings_section_callback(  ) {
		_e( "Repeat the links you used in your articles at the bottom of the post", 'linkpizza-manager' );
	}

	/**
	 *
	 * Adds linkPizza disabled categories section description
	 *
	 * Since 4.9.5
	 *
	 */

	function pzz_linkPizza_Page_section_help_callback(  ) {
		?>
		<div>
			<p><?php _e('Go directly to your most important pages', 'linkpizza-manager' );?></p>
			<ul>
				<li><a href="<?php echo get_admin_url().'widgets.php' ?>"><?php _e('LinkPizza Link widget','linkpizza-manager');?></a> - <?php _e('Add a LinkPizza Link widget to your sidebar or footer', 'linkpizza-manager' );?> </li>
				<li><a href="https://linkpizza.com/publisher/statistics"><?php _e('Statistics','linkpizza-manager');?></a> - <?php _e('Go to your statistics on our website','linkpizza-manager'); ?></li>
				<li><a href="http://support.linkpizza.com/"><?php _e('FAQ','linkpizza-manager');?></a> - <?php _e('Go to our frequently asked questions','linkpizza-manager'); ?> </li>
			</ul>
		</div>
		<div>
			<h2><?php _e('Shortcodes', 'linkpizza-manager' );?></h2>
			<p><?php _e('Currently LinkPizza has  a shortcode you can use in your posts to insert a widget', 'linkpizza-manager' );?></p>
			<ul>
				<li>[pzzwidget id=(<b><?php _e('ID of the widget, you can find these on ZEEF.com, looking at our examples or be provided one by your account manager', 'linkpizza-manager' ) ?></b>) width=(<b><?php _e('width of the widget', 'linkpizza-manager' ) ?></b>) height=(<b><?php _e('height of the widget', 'linkpizza-manager' ) ?></b>)]</li>
            </ul>
		</div>
		<?php
	}


	/**
	 *
	 * Adds linkPizza options section
	 *
	 * Since 4.5
	 *
	 */

	function render_pzz_options_page() {
		if ( isset ( $_GET['tab'] ) ) $this->pzz_admin_tabs($_GET['tab']); else $this->pzz_admin_tabs('general');

		?>
		<form action='options.php' method='post'>
			<?php
			if (isset ( $_GET['tab'] ) ) {
				$tab = $_GET['tab'];
				if($tab == 'link-summary') {
					settings_fields( 'linkPizza_Page_link_summary' );
					do_settings_sections( 'linkPizza_Page_link_summary' );
					submit_button();
				}
				if($tab == 'advanced') {
					settings_fields('linkPizza_Page_advanced');
					do_settings_sections('linkPizza_Page_advanced');
					submit_button();
				}
				if($tab == 'general') {
					settings_fields( 'linkPizza_Page' );
					do_settings_sections( 'linkPizza_Page' );
				}
				if($tab == 'help') {
					do_settings_sections( 'linkPizza_Page_help' );
				}
			}else {
				settings_fields( 'linkPizza_Page' );
				do_settings_sections( 'linkPizza_Page' );
				submit_button();
			}

			?>
		</form>
		<?php

	}


	function pzz_admin_tabs( $current = 'homepage' ) {
		$pzz_general_string = esc_html__( 'General', 'linkpizza-manager' );
		$pzz_link_summary_string = esc_html__( 'Link Summary', 'linkpizza-manager' );
		$pzz_advanced_string = esc_html__( 'Advanced', 'linkpizza-manager' );
		$pzz_help_string = esc_html__( 'Help', 'linkpizza-manager' );

		$tabs = array( 'general' => $pzz_general_string,'link-summary' => $pzz_link_summary_string, 'advanced' => $pzz_advanced_string,'help' => $pzz_help_string );
		echo '<div id="icon-themes" class="icon32"><br></div>';
		echo '<h2 class="nav-tab-wrapper">';
		foreach( $tabs as $tab => $name ){
			$class = ( $tab == $current ) ? ' nav-tab-active' : '';
			echo "<a class='nav-tab$class' href='?page=linkpizza-manager&tab=$tab'>$name</a>";

		}
		echo '</h2>';
	}

	/**
	 *
	 * Adds linkPizza menu
	 *
	 * Since 4.5
	 *
	 */

	function pzz_add_admin_menu(  ) {

		add_menu_page( 'LinkPizza', 'LinkPizza', 'manage_options', 'linkpizza-manager', array( &$this, 'render_pzz_options_page') ,plugins_url('/assets/Icon-grey.png', __FILE__ ) );

	}




	/**
	* Create meta box to be displayed on the post editor screen.
	*
	* @since     1.0.0
	*
	*/
	function linkPizza_add_post_meta_boxes() {
	$screens = array( 'post', 'page' );
		foreach ( $screens as $screen ) {
			add_meta_box(
			  'linkpizza-post-enable',      // Unique ID
			  esc_html__( 'LinkPizza', 'linkpizza-manager' ),    // Title
			  array( $this, 'linkPizza_post_enable_meta_box' ),
			  $screen,     // Admin page (or post type)
			  'normal',         // Context
			  'core',         // Priority
                array(
                    '__block_editor_compatible_meta_box' => true, // Explicitly set Gutenberg compatibility
                )
			);
		}
	$other_post_types = get_post_types(array('_builtin'=>false));
		foreach ( $other_post_types as $other_post_type ) {
			add_meta_box(
			  'linkpizza-post-enable',      // Unique ID
			  esc_html__( 'LinkPizza', 'linkpizza-manager' ),    // Title
			  array( $this, 'linkPizza_post_enable_meta_box' ),
			  $other_post_type,     // Admin page (or post type)
			  'advanced',         // Context
			  'low'         // Priority
			);
		}

	}



	function custom_hidden_meta_boxes( $hidden ) {
		if(!in_array(get_post_type(), array('post','page'))){
			$hidden[] = 'linkpizza-post-enable';
			return $hidden;
		}else{
			return $hidden;
		}
	}

	/**
	* Display the post meta box.
	*
	* @since     1.0.0
	*
	*/
	function linkPizza_post_enable_meta_box($object) { ?>

	  <?php
		wp_nonce_field( 'linkPizza_post_custom_box', 'linkPizza_post_enable_nonce' );
		$disabledDomains = get_post_meta($object->ID, '_linkpizza_disabled_domains', true);
		$trackingOnlyDomains = get_post_meta($object->ID, '_linkpizza_tracking_only_domains', true);
		$pzzLinkSummaryEnabledGlobally = get_option("pzz_link_summary_enabled", false);
	  ?>


	<div width="100%">
	    <b><?php _e( "General", 'linkpizza-manager' ); ?></b>
	    <p>
		<input class="widefat" type="checkbox" name="linkpizza-tracking-only" id="linkpizza-tracking-only" value="1" <?php checked( get_post_meta( $object->ID, '_linkPizza_tracking_only', true ), 1 ); ?> size="30" />
		<label for="linkpizza-disabled"><?php _e( "Only track statistics but don't change links on this page to affiliate links.", 'linkpizza-manager' ); ?></label></br>
        <input class="widefat" type="checkbox" name="linkpizza-disabled" id="linkpizza-disabled" value="1" <?php checked( get_post_meta( $object->ID, '_linkPizza_disabled', true ), 1 ); ?> size="30" />
        <label for="linkpizza-disabled"><?php _e( "Disable LinkPizza (automatic affiliate links and tracking) for this specific post or page.", 'linkpizza-manager' ); ?></label></br>
        <?php if ($pzzLinkSummaryEnabledGlobally) { ?>
            <input class="widefat" type="checkbox" name="pzz_link_summary_disabled_post" id="pzz_link_summary_disabled_post" value="1" <?php checked( get_post_meta( $object->ID, '_pzz_link_summary_disabled_post', true ), 1 ); ?> size="30" />
            <label for="linkpizza-disabled"><?php _e( "Disable link summary for this specific post or page.", 'linkpizza-manager' ); ?></label>
        <?php } ?>
	    </p>
	    <b><?php _e( "Turn affiliate off for specific links", 'linkpizza-manager' ); ?></b>
	    <p>
		<?php
			if (class_exists('DOMDocument')) {
				$dom_document = new DOMDocument();
                libxml_use_internal_errors(true);
                if ($object->post_content != ''){
				$dom_document->loadHTML($object->post_content);
				$dom_document->preserveWhiteSpace = false;

				//use DOMXpath to navigate the html with the DOM

				$elements = $dom_document->getElementsByTagName('a');
		        $domains = array();
				foreach ($elements as $link) {
				    if(!empty(parse_url($link->getAttribute('href'))['host']) && !in_array(parse_url($link->getAttribute('href'))['host'],$domains)) {
					    array_push($domains,parse_url($link->getAttribute('href'))['host']);
                    }
                }

					if (!is_null($domains)) {
					?>
					<table class="widefat fixed" cellspacing="2">

						<thead>
						<tr>
                            <th id="domain" scope="col"><?php _e("Domain", 'linkpizza-manager'); ?></th>
                            <th id="cb" scope="col"><?php _e("Tracking only *", 'linkpizza-manager'); ?></th>
                            <?php if($disabledDomains != 'empty' && !empty($disabledDomains)) { ?>
                            <th id="cb" scope="col"><?php _e("Disable", 'linkpizza-manager'); ?>*</th>
                            <?php } ?>
							<th id="domainMonetizable" scope="col"><?php _e("Monetizable **", 'linkpizza-manager'); ?></th>
						</tr>
						</thead>
						<?php
						foreach ($domains as $domain) {
							?>
							<tr>
                                <td>
									<?php
									echo $domain;
									?>
                                </td>
                                <td>
                                    <input class="widefat" type="checkbox" name="linkpizza-tracking-only-domains[]"
                                           value="<?php echo $domain ?>"
										<?php
										if (is_array($trackingOnlyDomains)){
										echo (in_array($domain, $trackingOnlyDomains)) ? 'checked="checked"' : ''; ?> /> <?php
									} else {
										if ($domain == $trackingOnlyDomains) {
											echo 'checked="checked" />';
										} else {
											echo '/>';
										}
									}
									?>
                                </td>
							<?php if($disabledDomains != 'empty' && !empty($disabledDomains)) { ?>

                                <td>
									<input class="widefat" type="checkbox" name="linkpizza-disabled-domains[]"
										   value="<?php echo $domain ?>"
										<?php
										if (is_array($disabledDomains)){
										echo (in_array($domain, $disabledDomains)) ? 'checked="checked"' : ''; ?> /> <?php
									} else {
										if ($domain == $disabledDomains) {
											echo 'checked="checked" />';
										} else {
											echo '/>';
										}
									}
									?>
								</td>
							<?php } ?>
                                <td>
									<?php
									$response = pzz_do_oauth_call_with_refresh_check(PZZ_OIDC_API_BASE_PATH.'/url/isMonetizable?url=http://' . $domain . '', array(), false);
									if ($response == 'true') {
										?> <img src="<?php echo plugins_url('/assets/dollar-sign-orange-small.png', __FILE__); ?>"
												alt="<?php _e("Monetized", 'linkpizza-manager'); ?>"
												title="<?php _e("Monetized", 'linkpizza-manager'); ?>"
												style="margin: -10px;padding-left: 25px;"/> <?php
									} else {
										?> <img src="<?php echo plugins_url('/assets/dollar-sign-small.png', __FILE__); ?>"
												alt="<?php _e("Not Monetized", 'linkpizza-manager'); ?>"
                                                title="<?php _e("Not Monetized", 'linkpizza-manager'); ?>"
												style="margin: -10px;padding-left: 25px;"/> <?php
									}
									?>
								</td>
							</tr>
							<?php
						}
						?>
					</table>
                    <div>
                        <span><?php _e("* Tracking only: Only track statistics but don't change this link to an affiliate link", 'linkpizza-manager'); ?></span>
                        <br/>
                        <span><?php _e("** Monetizable: LinkPizza can monetize this link automatically for you", 'linkpizza-manager'); ?></span>
                    </div>


					<?php
					}
					libxml_clear_errors();
				} else {
					?><?php _e("You haven't added any links to your blogpost, maybe you want to add some", 'linkpizza-manager'); ?>
					<a href="https://linkpizza.com/about/our-advertisers"
					   target="_blank"> <?php _e("advertiser links", 'linkpizza-manager'); ?></a>  <?php _e("to monetize this post?", 'linkpizza-manager'); ?><?php
				}
			}else{
				?>
				<div class="warning">
					<?php	_e("It seems your Wordpress is missing php-xml, please ask your hosting to install it. Monetized links are disabled untill fixed.", 'linkpizza-manager'); ?>
				</div>
				<?php
			}
			?>
	    </p>
	</div><?php
	 }


	/**
	 * Save the meta when the post is saved.
	 *
	 * @param int $post_id The ID of the post being saved.
	 */
	public function save( $post_id ) {

		/*
		 * We need to verify this came from the our screen and with proper authorization,
		 * because save_post can be triggered at other times.
		 */

		// Check if our nonce is set.
		if ( ! isset( $_POST['linkPizza_post_enable_nonce'] ) )
			return $post_id;

		$nonce = $_POST['linkPizza_post_enable_nonce'];

		// Verify that the nonce is valid.
		if ( ! wp_verify_nonce( $nonce, 'linkPizza_post_custom_box' ) )
			return $post_id;

		// If this is an autosave, our form has not been submitted,
		// so we don't want to do anything.
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )
			return $post_id;

		// Check the user's permissions.
		if ( 'page' == $_POST['post_type'] ) {

			if ( ! current_user_can( 'edit_page', $post_id ) )
				return $post_id;

		} else {

			if ( ! current_user_can( 'edit_post', $post_id ) )
				return $post_id;
		}

		/* OK, its safe for us to save the data now. */

		if ( isset( $_POST['linkpizza-disabled'] ) ) {
            $value =  '1';
        } else {
            $value = '';
        }

		if ( isset( $_POST['linkpizza-tracking-only'] ) ) {
			$tracking_only =  '1';
		} else {
			$tracking_only = '';
		}

		if ( isset( $_POST['pzz_link_summary_disabled_post'] ) ) {
            $value_link_summary =  '1';
        } else {
            $value_link_summary = '';
        }

		if(isset( $_POST['linkpizza-disabled-domains'] ))
		{
		    $custom = $_POST['linkpizza-disabled-domains'];
		    $old_meta = get_post_meta($post_id, '_linkpizza_disabled_domains', true);
		    // Update post meta
		    if(!empty($old_meta)){
			update_post_meta($post_id, '_linkpizza_disabled_domains', $custom);
		    } else {
			add_post_meta($post_id, '_linkpizza_disabled_domains', $custom, true);
		    }
		} else {
			update_post_meta($post_id, '_linkpizza_disabled_domains', 'empty');
        }
		if(isset( $_POST['linkpizza-tracking-only-domains'] ))
		{
			$custom = $_POST['linkpizza-tracking-only-domains'];
			$old_meta = get_post_meta($post_id, '_linkpizza_tracking_only_domains', true);
			// Update post meta
			if(!empty($old_meta)){
				update_post_meta($post_id, '_linkpizza_tracking_only_domains', $custom);
			} else {
				add_post_meta($post_id, '_linkpizza_tracking_only_domains', $custom, true);
			}
		} else {
			update_post_meta($post_id, '_linkpizza_tracking_only_domains', 'empty');
		}

		// Update the meta field.
		update_post_meta( $post_id, '_linkPizza_disabled', $value);
		update_post_meta( $post_id, '_linkPizza_tracking_only', $tracking_only);
		update_post_meta( $post_id, '_pzz_link_summary_disabled_post', $value_link_summary);
	}

	/**
	 * Register the scripts for the admin-facing side of the site.
	 *
	 * @since    4.5
	 */
	public function admin_enqueue_scripts() {
		wp_enqueue_script( 'chartist', '//cdn.jsdelivr.net/chartist.js/latest/chartist.min.js', array('jquery'),true);
		wp_enqueue_script( 'pzz-admin', plugins_url('/js/admin.js', __FILE__ ), array( 'wp-color-picker' ), false, true );
		wp_enqueue_script('jquery-ui-datepicker', array('jquery'),true);

	}

	/**
	 * Register the stylesheets for the admin-facing side of the site.
	 *
	 * @since    4.5
	 */
	public function admin_enqueue_style() {
		wp_enqueue_style( 'wp-color-picker' );
		wp_enqueue_style('jquery-ui-css', '//ajax.googleapis.com/ajax/libs/jqueryui/1.8.8/themes/smoothness/jquery-ui.css');
	}


	function pzz_increase_preview_nonce() {
		$extendPreviewTime = get_option('pzz_extend_preview_time');
	    if($extendPreviewTime == '1') {
		    return 60 * 60 * 24 * 30; // 30 days
	    } else {
		    return 60 * 60 * 24 * 2; // 48 Hours ( Default )
	    }
	}

	public static function plugin_row_meta($meta, $file) {
		if ($file == PZZ_BASE_FILE) {
			$meta[] = '<a href="admin.php?page=linkpizza-manager">' . __('Settings','linkpizza-manager') . '</a>';
			$meta[] = '<a href="https://linkpizza.com/signup" target="_blank">' . __('Sign up','linkpizza-manager') . '</a>';
		}
		return $meta;
	}

	/**
	 * Add column to pages/posts page showing is LinkPizza is visible
	 *
	 * @since    5.2
	 */

	public function pzz_add_posts_pages_list_column($columns) {
		$columns['linkpizza'] = 'LinkPizza';
		return $columns;
	}

	/**
	 * Render column to pages/posts page showing is LinkPizza is visible
	 *
	 * @since    5.2
	 */

	public function pzz_render_posts_pages_list_column($column_name, $id) {
		switch ($column_name) {
			case 'linkpizza':
				$linkpizza_disabled = get_post_meta($id, '_linkPizza_disabled', true);
				$linkpizza_tracking_only = get_post_meta($id, '_linkPizza_tracking_only', true);
				$disabledDomains = get_post_meta($id, '_linkpizza_disabled_domains', true);
				$trackingOnlyDomains = get_post_meta($id, '_linkpizza_tracking_only_domains', true);
				if($linkpizza_disabled || $linkpizza_tracking_only)
                    echo 'Off';
				else if(!empty($disabledDomains) && $disabledDomains != 'empty')
					echo 'disabled: '.json_encode( $disabledDomains );
                else if (!empty($trackingOnlyDomains) && $trackingOnlyDomains != 'empty')
	                echo 'tracking-only: '. json_encode( $trackingOnlyDomains );
				else echo 'On';
				break;
		}
	}

	/**
	 * register bulk action dropdown addon
	 *
	 * @since    5.2
	 */

	function register_pzz_bulk_actions($bulk_actions) {
		$bulk_actions['disable_linkpizza'] = __( 'Disable LinkPiza', 'linkpizza-manager');
		$bulk_actions['enable_linkpizza'] = __( '(Re)enable LinkPiza', 'linkpizza-manager');
		return $bulk_actions;
	}

	/**
	 * Handle bulk actions
	 *
	 * @since    5.2
	 */

	function pzz_bulk_action_handler( $redirect_to, $action_name, $post_ids ) {
		if ( 'disable_linkpizza' === $action_name ) {
			foreach ( $post_ids as $post_id ) {
				update_post_meta( $post_id, '_linkPizza_disabled', '1');
			}
			$redirect_to = add_query_arg( 'pzz_post_disabled', count( $post_ids ), $redirect_to );
			return $redirect_to;
		}

        elseif ( 'enable_linkpizza' === $action_name ) {
			foreach ( $post_ids as $post_id ) {
				update_post_meta( $post_id, '_linkPizza_disabled', '');
			}
			$redirect_to = add_query_arg( 'pzz_post_enabled', count( $post_ids ), $redirect_to );
			return $redirect_to;
		}

		else
			return $redirect_to;
	}

	function pzz_bulk_action_admin_notice() {
		if ( ! empty( $_REQUEST['pzz_post_disabled'] ) ) {
			$post_count = intval( $_REQUEST['pzz_post_disabled'] );
		    ?>
            <div class="updated">
                <p><?php esc_html_e( 'LinkPizza disabled on '.$post_count.' posts/pages', 'linkpizza-manager' ); ?></p>
            </div>
            <?php
		}
		if ( ! empty( $_REQUEST['pzz_post_enabled'] ) ) {
			$post_count = intval( $_REQUEST['pzz_post_enabled'] );
			?>
            <div class="updated">
                <p><?php esc_html_e( 'LinkPizza enabled on '.$post_count.' posts/pages', 'linkpizza-manager' ); ?></p>
            </div>
			<?php
		}
	}

}