<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://linkpizza.com
 * @since      1.0.0
 *
 * @package    linkPizza-Manager
 * @subpackage linkPizza-Manager/public
 */

class linkPizza_Manager_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the scripts for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {}


	/**
	 * Register the styles for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		wp_enqueue_style('pzz-public', plugin_dir_url(__FILE__) . 'css/pzz-public.css');
	}
	
	/**
	 * Add the javascript that loads LinkPizza asynchronously 
	 *
	 * @since    4.4.1
	 */
	public function add_linkpizza_head(){
		?><?php
		$postid = get_the_ID();
		$disabledDomains = get_post_meta($postid, '_linkpizza_disabled_domains', true);
		$trackingOnlyDomains = get_post_meta($postid, '_linkpizza_tracking_only_domains', true);
		$indexableJavascriptPath = '';
		$indexableJavascript = get_option('pzz_indexable_javascript');

		if($indexableJavascript == '1') {
		    $indexableJavascript = 'alt/';
        }
		?>

        <script>
	        <?php if( current_user_can('edit_posts')) { ?>
                //Logged in admin, do not measure views or reads
                window.pzzMeasureViews = false;
	       <?php } ?>
            <?php if(!empty($disabledDomains) && $disabledDomains != 'empty') {echo 'window.pzzIgnoredDomains ='.json_encode( $disabledDomains ).';'; } ?>
	        <?php if(!empty($trackingOnlyDomains) && $trackingOnlyDomains != 'empty') {echo 'window.pzzTrackingOnlyDomains ='.json_encode( $trackingOnlyDomains ).';'; } ?>

            (function(p,z,Z){
                z=p.createElement("script");z.async=1;
                z.src="//pzz.io/<?php echo $indexableJavascript ?>pzz"+".js?uid=<?php $user_id = get_option( 'pzz_id' ); echo $user_id; ?>&host="+p.domain;
                (p.head||p.documentElement).insertBefore(z,Z);
            })(document);
        </script>
        <?php
	}

	public function add_linkpizza_body_classes($classes) {
		$postid = get_the_ID();
		$linkpizza_disabled = get_post_meta($postid, '_linkPizza_disabled', true);
		$linkpizza_tracking_only = get_post_meta($postid, '_linkPizza_tracking_only', true);
		$trackingOnlyCategories = get_option('pzz_tracking_only_categories');
		$trackingBeforeBoolean = 0;
		$trackingOnlyBefore = get_option('pzz_tracking_only_on_posts_before');
		if( strtotime($trackingOnlyBefore) > strtotime(get_the_date('m/d/Y', $postid )) ) {
			$trackingBeforeBoolean = 1;
		}

		$categories =  get_the_category();
		$isTrackingOnlyCategory = false;

		//loop through categories to check if its enabled
		if (!empty($categories) && !empty($trackingOnlyCategories)) {
			if (is_array($categories)) {
				foreach ($categories as $category) {
					if (in_array($category->cat_ID, $trackingOnlyCategories)) {
						$isTrackingOnlyCategory = true;
					}
				}
			} else {
				if (in_array($categories->cat_ID, $trackingOnlyCategories)) {
					$isTrackingOnlyCategory = true;
				}
			}
		}

		if ($linkpizza_tracking_only == '1' || $isTrackingOnlyCategory || $trackingBeforeBoolean == 1) {
		    array_push($classes, 'pzz-tracking-only');
		    if ($linkpizza_tracking_only == '1') {
			    array_push($classes, 'pzz-post-is-tracking-only');
		    }
			if ($isTrackingOnlyCategory) {
				array_push($classes, 'pzz-category-is-tracking-only');
			}
			if ($trackingBeforeBoolean == 1) {
				array_push($classes, 'pzz-tracking-only-date');
			}

		}

		if ($linkpizza_disabled == '1') {
			array_push($classes, 'pzz-ignore');
			array_push($classes, 'pzz-post-is-disabled');
		}

		return $classes;

    }

	public function pzz_add_link_summary($content)
	{
		$postid = get_the_ID();
		$pzz_link_summary_enabled = get_option('pzz_link_summary_enabled');
		$pzz_link_summary_disabled_post = get_post_meta($postid, '_pzz_link_summary_disabled_post', true);

		if ($pzz_link_summary_disabled_post != '1' && $pzz_link_summary_enabled == '1') {
			$pzz_link_summary_border_color = get_option('pzz_link_summary_border_color');
			$pzz_link_summary_border_width = get_option('pzz_link_summary_border_width');
			$pzz_link_summary_border_padding = get_option('pzz_link_summary_border_padding');
			$pzz_link_summary_width = get_option('pzz_link_summary_width');
			$pzz_link_summary_link_color = get_option('pzz_link_summary_link_color');
			$pzz_link_summary_layout_type = get_option('pzz_link_summary_layout_type');
			$pzz_link_summary_position = get_option('pzz_link_summary_position');
			$pzz_link_summary_tag_background_color = get_option('pzz_link_summary_tag_background_color');
			$pzz_link_summary_use_text_title = get_option('pzz_link_summary_use_title_text');

			if (class_exists('DOMDocument')) {
				libxml_use_internal_errors(true);
				$dom_document = new DOMDocument();
				if ($content != '') {
					if ($dom_document->loadHTML($content)) {
						$dom_document->preserveWhiteSpace = false;

						//use DOMXpath to navigate the html with the DOM

						$elements = $dom_document->getElementsByTagName('a');

						if (!($elements->length == 0)) {
							$pzz_tags_background_style = '';
							if ($pzz_link_summary_layout_type == 1) {
								$pzz_layout_ol_style = '';
								$pzz_class = 'pzz-link-summary-list';
								switch ($pzz_link_summary_position) {
									case 1:
										$pzz_layout_style = 'text-align: left; border: ' . $pzz_link_summary_border_width . 'px solid ' . $pzz_link_summary_border_color . '; width:' . $pzz_link_summary_width . '%; padding: ' . $pzz_link_summary_border_padding . 'px;  ';
										break;
									case 2:
										$pzz_layout_style = 'text-align: center; border: ' . $pzz_link_summary_border_width . 'px solid ' . $pzz_link_summary_border_color . '; width:' . $pzz_link_summary_width . '%; padding: ' . $pzz_link_summary_border_padding . 'px; margin: 0 auto; ';
										break;
									case 3:
										$pzz_layout_style = 'text-align: left; border: ' . $pzz_link_summary_border_width . 'px solid ' . $pzz_link_summary_border_color . '; width:' . $pzz_link_summary_width . '%; padding: ' . $pzz_link_summary_border_padding . 'px; margin: 0 ' . (100 - $pzz_link_summary_width) . '%; ';
										break;
								}
							} else {
								$pzz_class = 'pzz-link-summary-tags';
								$pzz_layout_style = '';
								switch ($pzz_link_summary_position) {
									case 1:
										$pzz_layout_ol_style = 'text-align: left;';
										break;
									case 2:
										$pzz_layout_ol_style = 'text-align: center; ';
										break;
									case 3:
										$pzz_layout_ol_style = 'text-align: right;';
										break;
								}
								if (empty($pzz_link_summary_tag_background_color) || $pzz_link_summary_tag_background_color == 1) {
									$pzz_link_summary_tag_background_color = 'transparent';
								}
								$pzz_tags_background_style = 'style="background-color:' . $pzz_link_summary_tag_background_color . ';"';
							}
							$aftercontent = '<div class="' . $pzz_class . '" style="' . $pzz_layout_style . '"><h5>' . __('Links in this article', 'linkpizza-manager') . '</h5><ol style="' . $pzz_layout_ol_style . '">';
							foreach ($elements as $element) {
								$target_url = $element->getAttribute('href');
								$domain = parse_url($element->getAttribute('href'), PHP_URL_HOST);
								$stripped_domain = preg_replace('/^www\./', '', $domain);
								$text_title = $element->textContent;
								$title_attribute = $element->getAttribute('title');
								if (!empty($title_attribute) && $title_attribute != '' && $pzz_link_summary_use_text_title == 1) {
									$aftercontent = $aftercontent . '<li ' . $pzz_tags_background_style . '><a style="color:' . $pzz_link_summary_link_color . ';" href="' . $target_url . '"">' . $title_attribute . '</a></li>';
								} else {
									if (!empty($text_title) && $text_title != '' && $pzz_link_summary_use_text_title == 1) {
										$aftercontent = $aftercontent . '<li ' . $pzz_tags_background_style . '><a style="color:' . $pzz_link_summary_link_color . ';" href="' . $target_url . '"">' . $text_title . '</a></li>';
									} else {
										$aftercontent = $aftercontent . '<li ' . $pzz_tags_background_style . '><a style="color:' . $pzz_link_summary_link_color . ';" href="' . $target_url . '"">' . $stripped_domain . '</a></li>';
									}
								}
							}
							$aftercontent = $aftercontent . '</ol></div>';
							$fullcontent = $content . $aftercontent;
							return $fullcontent;
						}
						return $content;
					}
					return $content;
				}
				libxml_clear_errors();
				return $content;
			}
		}
		return $content;
	}

}
