<?php


/**
 *
 * Try to do an API call to LinkPizza, if the token is invalid which is represented by 404 in the API
 * retrieve new token and try again. Fail after 1 retry.
 *
 * Since 4.7
 *
 */

function pzz_do_oauth_call_with_refresh_check ($url,$parameters,$retries){
    if($retries < 2 ){
        $client = new OAuth2\Client(PZZ_OIDC_CLIENT_ID, '', OAuth2\Client::AUTH_TYPE_URI, ABSPATH . WPINC . '/certificates/ca-bundle.crt');
        $accessOidcToken = get_option(PZZ_OIDC_ACCESS_TOKEN_OPTION_NAME);
        try{
            $client->setAccessToken($accessOidcToken);
            $client->setAccessTokenType(1);
            $response = $client->fetch($url,$parameters);
            if($response['code'] == 200){
                return $response['result'];
            }else{
                $refreshOidcToken = get_option(PZZ_OIDC_REFRESH_TOKEN_OPTION_NAME);
                if(($response['code'] == 401 || $response['code'] == 403) && $refreshOidcToken != ''){
                    $callbackUrl = admin_url( 'admin.php?page=linkpizza-manager&pzz_callback=oauth');
                    $params = array('refresh_token' => $refreshOidcToken, 'redirect_uri' => $callbackUrl);
                    $extraHeaders = array('Host' => PZZ_OIDC_HOST_HEADER);
                    $response = $client->getAccessToken(PZZ_OIDC_TOKEN_ENDPOINT, 'refresh_token', $params, $extraHeaders);
                    if ($response['code'] == 200 && isset($response["result"]) && isset($response["result"]["access_token"])) {
                        $accessOidcToken = $response["result"]["access_token"];
                        $refreshOidcToken = $response["result"]["refresh_token"];

                        if (get_option(PZZ_OIDC_ACCESS_TOKEN_OPTION_NAME) !== false) {
                            update_option(PZZ_OIDC_ACCESS_TOKEN_OPTION_NAME, $accessOidcToken, 'yes');
                        } else {
                            add_option(PZZ_OIDC_ACCESS_TOKEN_OPTION_NAME, $accessOidcToken, '', 'yes');
                        }

                        if (get_option(PZZ_OIDC_REFRESH_TOKEN_OPTION_NAME) !== false) {
                            update_option(PZZ_OIDC_REFRESH_TOKEN_OPTION_NAME, $refreshOidcToken, 'yes');
                        } else {
                            add_option(PZZ_OIDC_REFRESH_TOKEN_OPTION_NAME, $refreshOidcToken, '', 'yes');
                        }
                        pzz_do_oauth_call_with_refresh_check($url, $parameters, $retries + 1);
                    }else{
                        update_option(PZZ_OIDC_ACCESS_TOKEN_OPTION_NAME, "", 'yes');
                        update_option(PZZ_OIDC_REFRESH_TOKEN_OPTION_NAME, "", 'yes');
                        pzz_write_log( 'Silently failed to connect to the LinkPizza API: Access token not present' );
                    }
                }
            }
        }catch (\OAuth2\Exception $e){
            echo 'Caught exception: ',  $e->getMessage(), "\n";
        }
    }else{
        pzz_write_log( 'Silently failed to connect to the LinkPizza API: kept getting 404s' );
    }
}

/**
 *
 * Small convenience function to write to the WordPress error log
 *
 * Since 4.7
 *
 */

function pzz_write_log ( $log )  {
    if ( true === WP_DEBUG ) {
        if ( is_array( $log ) || is_object( $log ) ) {
            error_log( print_r( $log, true ) );
        } else {
            error_log( $log );
        }
    }
}